@@ -1,4 +1,4 @@
-/* Copyright (C) 2007-2010 Open Information Security Foundation
+/* Copyright (C) 2007-2012 Open Information Security Foundation
  *
  * You can copy, redistribute or modify this Program under the terms of
  * the GNU General Public License version 2 as published by the Free
@@ -50,11 +50,16 @@
 #include "stream-tcp-reassemble.h"
 #include "util-host-os-info.h"
 
+#include "defrag.h"
+#include "defrag-hash.h"
+#include "defrag-queue.h"
+
 #ifdef UNITTESTS
 #include "util-unittest.h"
 #endif
 
 #define DEFAULT_DEFRAG_HASH_SIZE 0xffff
+#define DEFAULT_DEFRAG_POOL_SIZE 0xffff
 
 /**
  * Default timeout (in seconds) before a defragmentation tracker will
@@ -87,113 +92,6 @@ enum defrag_policies {
 
 static int default_policy = DEFRAG_POLICY_BSD;
 
-/**
- * A context for an instance of a fragmentation re-assembler, in case
- * we ever need more than one.
- */
-typedef struct DefragContext_ {
-    uint64_t ip4_frags; /**< Number of IPv4 fragments seen. */
-    uint64_t ip6_frags; /**< Number of IPv6 fragments seen. */
-
-    HashListTable *frag_table; /**< Hash (list) table of fragment trackers. */
-    SCMutex frag_table_lock;
-
-    Pool *tracker_pool; /**< Pool of trackers. */
-    SCMutex tracker_pool_lock;
-
-    Pool *frag_pool; /**< Pool of fragments. */
-    SCMutex frag_pool_lock;
-
-    time_t timeout; /**< Default timeout. */
-} DefragContext;
-
-/**
- * Storage for an individual fragment.
- */
-typedef struct Frag_ {
-    DefragContext *dc; /**< The defragmentation context this frag was
-                        * allocated under. */
-
-    uint16_t offset; /**< The offset of this fragment, already
-                      * multiplied by 8. */
-
-    uint16_t len; /**< The length of this fragment. */
-
-    uint8_t hlen; /**< The length of this fragments IP header. */
-
-    uint8_t more_frags; /**< More frags? */
-
-    uint16_t ip_hdr_offset; /**< Offset in the packet where the IP
-                               * header starts. */
-    uint16_t frag_hdr_offset; /**< Offset in the packet where the frag
-                               * header starts. */
-
-    uint16_t data_offset; /**< Offset to the packet data. */
-    uint16_t data_len; /**< Length of data. */
-
-    uint8_t *pkt; /**< The actual packet. */
-
-    uint16_t ltrim; /**< Number of leading bytes to trim when
-                     * re-assembling the packet. */
-
-    int8_t skip; /**< Skip this fragment during re-assembly. */
-
-#ifdef DEBUG
-    uint64_t pcap_cnt;  /* pcap_cnt of original packet */
-#endif
-
-    TAILQ_ENTRY(Frag_) next; /**< Pointer to next fragment for tailq. */
-} Frag;
-
-/** \brief Reset tracker fields except "dc" and "lock" */
-#define DEFRAG_TRACKER_RESET(t) { \
-    (t)->timeout = 0; \
-    (t)->id = 0; \
-    (t)->policy = 0; \
-    (t)->af = 0; \
-    (t)->seen_last = 0; \
-    CLEAR_ADDR(&(t)->src_addr); \
-    CLEAR_ADDR(&(t)->dst_addr); \
-    (t)->frags.tqh_first = NULL; \
-    (t)->frags.tqh_last = NULL; \
-}
-
-/**
- * A defragmentation tracker.  Used to track fragments that make up a
- * single packet.
- */
-typedef struct DefragTracker_ {
-    SCMutex lock; /**< Mutex for locking list operations on
-                           * this tracker. */
-
-    DefragContext *dc; /**< The defragmentation context this tracker
-                        * was allocated under. */
-
-    uint32_t timeout; /**< When this tracker will timeout. */
-
-    uint32_t id; /**< IP ID for this tracker.  32 bits for IPv6, 16
-                  * for IPv4. */
-
-    uint8_t policy; /**< Reassembly policy this tracker will use. */
-
-    uint8_t af; /**< Address family for this tracker, AF_INET or
-                 * AF_INET6. */
-
-    uint8_t seen_last; /**< Has this tracker seen the last fragment? */
-
-    Address src_addr; /**< Source address for this tracker. */
-    Address dst_addr; /**< Destination address for this tracker. */
-
-    TAILQ_HEAD(frag_tailq, Frag_) frags; /**< Head of list of fragments. */
-} DefragTracker;
-
-/** A random value used for hash key generation. */
-static int defrag_hash_rand;
-
-/** Hash table size, and also the maximum number of trackers that will
- * be allocated. */
-static int defrag_hash_size;
-
 /** The global DefragContext so all threads operate from the same
  * context. */
 static DefragContext *defrag_context;
@@ -218,187 +116,49 @@ DumpFrags(DefragTracker *tracker)
 #endif /* UNITTESTS */
 #endif
 
-/**
- * Generate a key for looking of a fragtracker in a hash
- * table. Adapted from the hash function in flow-hash.c.
- *
- * \todo Test performance and distribution.
- */
-static uint32_t
-DefragHashFunc(HashListTable *ht, void *data, uint16_t datalen)
-{
-    DefragTracker *p = (DefragTracker *)data;
-    uint32_t key;
-
-    if (p->af == AF_INET) {
-        key = (defrag_hash_rand + p->id +
-            p->src_addr.addr_data32[0] + p->dst_addr.addr_data32[0]) %
-            defrag_hash_size;
-    }
-    else if (p->af == AF_INET6) {
-        key = (defrag_hash_rand + p->id +
-            p->src_addr.addr_data32[0] + p->src_addr.addr_data32[1] +
-            p->src_addr.addr_data32[2] + p->src_addr.addr_data32[3] +
-            p->dst_addr.addr_data32[0] + p->dst_addr.addr_data32[1] +
-            p->dst_addr.addr_data32[2] + p->dst_addr.addr_data32[3]) %
-            defrag_hash_size;
-    }
-    else
-        key = 0;
-
-    return key;
-}
-
-/**
- * \brief Compare 2 DefragTracker nodes in case of hash conflict.
- *
- * \retval 1 if a and b match, otherwise 0.
- */
-static char
-DefragHashCompare(void *a, uint16_t a_len, void *b, uint16_t b_len)
-{
-    DefragTracker *dta = (DefragTracker *)a;
-    DefragTracker *dtb = (DefragTracker *)b;
-
-    if (dta->af != dtb->af)
-        return 0;
-    else if (dta->id != dtb->id)
-        return 0;
-    else if (!CMP_ADDR(&dta->src_addr, &dtb->src_addr))
-        return 0;
-    else if (!CMP_ADDR(&dta->dst_addr, &dtb->dst_addr))
-        return 0;
-
-    /* Match. */
-    return 1;
-}
-
-/**
- * \brief Called by the hash table when a tracker is removed from the
- *     hash table.
- *
- * We don't actually do anything here.  The tracker will be reset and
- * put back into a memory pool.
- */
-static void
-DefragHashFree(void *data)
-{
-}
-
 /**
  * \brief Reset a frag for reuse in a pool.
  */
 static void
 DefragFragReset(Frag *frag)
 {
-    DefragContext *dc = frag->dc;
-
     if (frag->pkt != NULL)
         SCFree(frag->pkt);
     memset(frag, 0, sizeof(*frag));
-    frag->dc = dc;
 }
 
 /**
  * \brief Allocate a new frag for use in a pool.
  */
-static void *
-DefragFragNew(void *arg)
+static int
+DefragFragInit(void *data, void *initdata)
 {
-    DefragContext *dc = arg;
-    Frag *frag;
+    Frag *frag = data;
 
-    frag = SCCalloc(1, sizeof(*frag));
-    if (frag == NULL)
-        return NULL;
-    frag->dc = dc;
-
-    return (void *)frag;
-}
-
-/**
- * \brief Free a frag when released from a pool.
- */
-static void
-DefragFragFree(void *arg)
-{
-    Frag *frag = arg;
-    SCFree(frag);
+    memset(frag, 0, sizeof(*frag));
+    return 1;
 }
 
 /**
  * \brief Free all frags associated with a tracker.
  */
-static void
+void
 DefragTrackerFreeFrags(DefragTracker *tracker)
 {
     Frag *frag;
 
     /* Lock the frag pool as we'll be return items to it. */
-    SCMutexLock(&tracker->dc->frag_pool_lock);
+    SCMutexLock(&defrag_context->frag_pool_lock);
 
     while ((frag = TAILQ_FIRST(&tracker->frags)) != NULL) {
         TAILQ_REMOVE(&tracker->frags, frag, next);
 
         /* Don't SCFree the frag, just give it back to its pool. */
         DefragFragReset(frag);
-        PoolReturn(frag->dc->frag_pool, frag);
+        PoolReturn(defrag_context->frag_pool, frag);
     }
 
-    SCMutexUnlock(&tracker->dc->frag_pool_lock);
-}
-
-/**
- * \brief Reset a tracker for reuse.
- */
-static void
-DefragTrackerReset(DefragTracker *tracker)
-{
-    DefragTrackerFreeFrags(tracker);
-    DEFRAG_TRACKER_RESET(tracker);
-    TAILQ_INIT(&tracker->frags);
-}
-
-/**
- * \brief Allocates a new defragmentation tracker for use in the pool
- *     for trackers.
- *
- * \arg Pointer to DefragContext this new tracker will be associated
- *     with.
- *
- * \retval A new DefragTracker if successfull, NULL on failure.
- */
-static void *
-DefragTrackerNew(void *arg)
-{
-    DefragContext *dc = arg;
-    DefragTracker *tracker;
-
-    tracker = SCCalloc(1, sizeof(*tracker));
-    if (tracker == NULL)
-        return NULL;
-    if (SCMutexInit(&tracker->lock, NULL) != 0) {
-        SCFree(tracker);
-        return NULL;
-    }
-    tracker->dc = dc;
-    TAILQ_INIT(&tracker->frags);
-
-    return (void *)tracker;
-}
-
-/**
- * \brief Free a defragmentation tracker that is being removed from
- *     the pool.
- */
-static void
-DefragTrackerFree(void *arg)
-{
-    DefragTracker *tracker = arg;
-
-    SCMutexDestroy(&tracker->lock);
-    DefragTrackerFreeFrags(tracker);
-    SCFree(tracker);
+    SCMutexUnlock(&defrag_context->frag_pool_lock);
 }
 
 /**
@@ -413,46 +173,24 @@ DefragContextNew(void)
     DefragContext *dc;
 
     dc = SCCalloc(1, sizeof(*dc));
-    if (dc == NULL)
+    if (unlikely(dc == NULL))
         return NULL;
 
-    /* Initialize the hash table. */
-    dc->frag_table = HashListTableInit(DEFAULT_DEFRAG_HASH_SIZE, DefragHashFunc,
-        DefragHashCompare, DefragHashFree);
-    if (dc->frag_table == NULL) {
-        SCLogError(SC_ERR_MEM_ALLOC,
-            "Defrag: Failed to initialize hash table.");
-        exit(EXIT_FAILURE);
-    }
-    if (SCMutexInit(&dc->frag_table_lock, NULL) != 0) {
-        SCLogError(SC_ERR_MEM_ALLOC,
-            "Defrag: Failed to initialize hash table mutex.");
-        exit(EXIT_FAILURE);
-    }
-
     /* Initialize the pool of trackers. */
     intmax_t tracker_pool_size;
-    if (!ConfGetInt("defrag.trackers", &tracker_pool_size)) {
+    if (!ConfGetInt("defrag.trackers", &tracker_pool_size) || tracker_pool_size == 0) {
         tracker_pool_size = DEFAULT_DEFRAG_HASH_SIZE;
     }
-    dc->tracker_pool = PoolInit(tracker_pool_size, tracker_pool_size,
-        DefragTrackerNew, dc, DefragTrackerFree);
-    if (dc->tracker_pool == NULL) {
-        SCLogError(SC_ERR_MEM_ALLOC,
-            "Defrag: Failed to initialize tracker pool.");
-        exit(EXIT_FAILURE);
-    }
-    if (SCMutexInit(&dc->tracker_pool_lock, NULL) != 0) {
-        SCLogError(SC_ERR_MUTEX,
-            "Defrag: Failed to initialize tracker pool mutex.");
-        exit(EXIT_FAILURE);
-    }
 
     /* Initialize the pool of frags. */
-    int frag_pool_size = 0xffff;
-    int frag_pool_prealloc = frag_pool_size / 4;
+    intmax_t frag_pool_size;
+    if (!ConfGetInt("defrag.max-frags", &frag_pool_size) || frag_pool_size == 0) {
+        frag_pool_size = DEFAULT_DEFRAG_POOL_SIZE;
+    }
+    intmax_t frag_pool_prealloc = frag_pool_size / 2;
     dc->frag_pool = PoolInit(frag_pool_size, frag_pool_prealloc,
-        DefragFragNew, dc, DefragFragFree);
+        sizeof(Frag),
+        NULL, DefragFragInit, dc, NULL, NULL);
     if (dc->frag_pool == NULL) {
         SCLogError(SC_ERR_MEM_ALLOC,
             "Defrag: Failed to initialize fragment pool.");
@@ -487,8 +225,8 @@ DefragContextNew(void)
     SCLogDebug("\tTimeout: %"PRIuMAX, (uintmax_t)dc->timeout);
     SCLogDebug("\tMaximum defrag trackers: %"PRIuMAX, tracker_pool_size);
     SCLogDebug("\tPreallocated defrag trackers: %"PRIuMAX, tracker_pool_size);
-    SCLogDebug("\tMaximum fragments: %d", frag_pool_size);
-    SCLogDebug("\tPreallocated fragments: %d", frag_pool_prealloc);
+    SCLogDebug("\tMaximum fragments: %"PRIuMAX, (uintmax_t)frag_pool_size);
+    SCLogDebug("\tPreallocated fragments: %"PRIuMAX, (uintmax_t)frag_pool_prealloc);
 
     return dc;
 }
@@ -499,9 +237,7 @@ DefragContextDestroy(DefragContext *dc)
     if (dc == NULL)
         return;
 
-    HashListTableFree(dc->frag_table);
     PoolFree(dc->frag_pool);
-    PoolFree(dc->tracker_pool);
     SCFree(dc);
 }
 
@@ -511,8 +247,7 @@ DefragContextDestroy(DefragContext *dc)
  * \param tracker The defragmentation tracker to reassemble from.
  */
 static Packet *
-Defrag4Reassemble(ThreadVars *tv, DefragContext *dc, DefragTracker *tracker,
-    Packet *p)
+Defrag4Reassemble(ThreadVars *tv, DefragTracker *tracker, Packet *p)
 {
     Packet *rp = NULL;
 
@@ -554,7 +289,7 @@ Defrag4Reassemble(ThreadVars *tv, DefragContext *dc, DefragTracker *tracker,
                    "fragmentation re-assembly, dumping fragments.");
         goto remove_tracker;
     }
-    SCLogDebug("Packet rp %p, p %p, rp->root %p", rp, p, rp->root);
+    PKT_SET_SRC(rp, PKT_SRC_DEFRAG);
     rp->recursion_level = p->recursion_level;
 
     int fragmentable_offset = 0;
@@ -612,15 +347,9 @@ Defrag4Reassemble(ThreadVars *tv, DefragContext *dc, DefragTracker *tracker,
     SET_PKT_LEN(rp, ip_hdr_offset + hlen + fragmentable_len);
 
 remove_tracker:
-    /* Remove the frag tracker. */
-    SCMutexLock(&dc->frag_table_lock);
-    HashListTableRemove(dc->frag_table, tracker, HASHLIST_NO_SIZE);
-    SCMutexUnlock(&dc->frag_table_lock);
-    DefragTrackerReset(tracker);
-    SCMutexLock(&dc->tracker_pool_lock);
-    PoolReturn(dc->tracker_pool, tracker);
-    SCMutexUnlock(&dc->tracker_pool_lock);
-
+    /** \todo check locking */
+    tracker->remove = 1;
+    DefragTrackerFreeFrags(tracker);
 done:
     return rp;
 }
@@ -631,8 +360,7 @@ done:
  * \param tracker The defragmentation tracker to reassemble from.
  */
 static Packet *
-Defrag6Reassemble(ThreadVars *tv, DefragContext *dc, DefragTracker *tracker,
-    Packet *p)
+Defrag6Reassemble(ThreadVars *tv, DefragTracker *tracker, Packet *p)
 {
     Packet *rp = NULL;
 
@@ -675,6 +403,7 @@ Defrag6Reassemble(ThreadVars *tv, DefragContext *dc, DefragTracker *tracker,
                 "fragmentation re-assembly, dumping fragments.");
         goto remove_tracker;
     }
+    PKT_SET_SRC(rp, PKT_SRC_DEFRAG);
 
     int fragmentable_offset = 0;
     int fragmentable_len = 0;
@@ -723,15 +452,9 @@ Defrag6Reassemble(ThreadVars *tv, DefragContext *dc, DefragTracker *tracker,
     SET_PKT_LEN(rp, ip_hdr_offset + sizeof(IPV6Hdr) + fragmentable_len);
 
 remove_tracker:
-    /* Remove the frag tracker. */
-    SCMutexLock(&dc->frag_table_lock);
-    HashListTableRemove(dc->frag_table, tracker, HASHLIST_NO_SIZE);
-    SCMutexUnlock(&dc->frag_table_lock);
-    DefragTrackerReset(tracker);
-    SCMutexLock(&dc->tracker_pool_lock);
-    PoolReturn(dc->tracker_pool, tracker);
-    SCMutexUnlock(&dc->tracker_pool_lock);
-
+    /** \todo check locking */
+    tracker->remove = 1;
+    DefragTrackerFreeFrags(tracker);
 done:
     return rp;
 }
@@ -742,8 +465,7 @@ done:
  * \todo Allocate packet buffers from a pool.
  */
 static Packet *
-DefragInsertFrag(ThreadVars *tv, DecodeThreadVars *dtv, DefragContext *dc,
-    DefragTracker *tracker, Packet *p)
+DefragInsertFrag(ThreadVars *tv, DecodeThreadVars *dtv, DefragTracker *tracker, Packet *p)
 {
     Packet *r = NULL;
     int ltrim = 0;
@@ -818,11 +540,8 @@ DefragInsertFrag(ThreadVars *tv, DecodeThreadVars *dtv, DefragContext *dc,
         return NULL;
     }
 
-    /* Lock this tracker as we'll be doing list operations on it. */
-    SCMutexLock(&tracker->lock);
-
     /* Update timeout. */
-    tracker->timeout = p->ts.tv_sec + dc->timeout;
+    tracker->timeout = p->ts.tv_sec + defrag_context->timeout;
 
     Frag *prev = NULL, *next;
     int overlap = 0;
@@ -928,21 +647,36 @@ DefragInsertFrag(ThreadVars *tv, DecodeThreadVars *dtv, DefragContext *dc,
 
 insert:
     if (data_len - ltrim <= 0) {
+        if (af == AF_INET) {
+            ENGINE_SET_EVENT(p, IPV4_FRAG_TOO_LARGE);
+        } else {
+            ENGINE_SET_EVENT(p, IPV6_FRAG_TOO_LARGE);
+        }
         goto done;
     }
 
     /* Allocate fragment and insert. */
-    SCMutexLock(&dc->frag_pool_lock);
-    Frag *new = PoolGet(dc->frag_pool);
-    SCMutexUnlock(&dc->frag_pool_lock);
+    SCMutexLock(&defrag_context->frag_pool_lock);
+    Frag *new = PoolGet(defrag_context->frag_pool);
+    SCMutexUnlock(&defrag_context->frag_pool_lock);
     if (new == NULL) {
+        if (af == AF_INET) {
+            ENGINE_SET_EVENT(p, IPV4_FRAG_IGNORED);
+        } else {
+            ENGINE_SET_EVENT(p, IPV6_FRAG_IGNORED);
+        }
         goto done;
     }
     new->pkt = SCMalloc(GET_PKT_LEN(p));
     if (new->pkt == NULL) {
-        SCMutexLock(&dc->frag_pool_lock);
-        PoolReturn(dc->frag_pool, new);
-        SCMutexUnlock(&dc->frag_pool_lock);
+        SCMutexLock(&defrag_context->frag_pool_lock);
+        PoolReturn(defrag_context->frag_pool, new);
+        SCMutexUnlock(&defrag_context->frag_pool_lock);
+        if (af == AF_INET) {
+            ENGINE_SET_EVENT(p, IPV4_FRAG_IGNORED);
+        } else {
+            ENGINE_SET_EVENT(p, IPV6_FRAG_IGNORED);
+        }
         goto done;
     }
     memcpy(new->pkt, GET_PKT_DATA(p) + ltrim, GET_PKT_LEN(p) - ltrim);
@@ -975,14 +709,14 @@ insert:
 
     if (tracker->seen_last) {
         if (tracker->af == AF_INET) {
-            r = Defrag4Reassemble(tv, dc, tracker, p);
+            r = Defrag4Reassemble(tv, tracker, p);
             if (r != NULL && tv != NULL && dtv != NULL) {
                 SCPerfCounterIncr(dtv->counter_defrag_ipv4_reassembled,
                     tv->sc_perf_pca);
             }
         }
         else if (tracker->af == AF_INET6) {
-            r = Defrag6Reassemble(tv, dc, tracker, p);
+            r = Defrag6Reassemble(tv, tracker, p);
             if (r != NULL && tv != NULL && dtv != NULL) {
                 SCPerfCounterIncr(dtv->counter_defrag_ipv6_reassembled,
                     tv->sc_perf_pca);
@@ -999,53 +733,9 @@ done:
             ENGINE_SET_EVENT(p, IPV6_FRAG_OVERLAP);
         }
     }
-    SCMutexUnlock(&tracker->lock);
     return r;
 }
 
-/**
- * \brief Timeout a tracker.
- *
- * Called when we fail to get a tracker from the pool.  The first
- * tracker that has expired will be released back to the pool then the
- * function will exit.
- *
- * Intended to be called with the tracker pool already locked.
- *
- * \param dc Current DefragContext.
- * \param p Packet that triggered this timeout run, used for timestamp.
- */
-static void
-DefragTimeoutTracker(ThreadVars *tv, DecodeThreadVars *dtv, DefragContext *dc,
-    Packet *p)
-{
-    HashListTableBucket *next = HashListTableGetListHead(dc->frag_table);
-    DefragTracker *tracker;
-    while (next != NULL) {
-        tracker = HashListTableGetListData(next);
-
-        if (tracker->timeout < (unsigned int)p->ts.tv_sec) {
-            /* Tracker has timeout out. */
-            HashListTableRemove(dc->frag_table, tracker, HASHLIST_NO_SIZE);
-            DefragTrackerReset(tracker);
-            PoolReturn(dc->tracker_pool, tracker);
-            if (tv != NULL && dtv != NULL) {
-                if (tracker->af == AF_INET) {
-                    SCPerfCounterIncr(dtv->counter_defrag_ipv4_timeouts,
-                        tv->sc_perf_pca);
-                }
-                else if (tracker->af == AF_INET6) {
-                    SCPerfCounterIncr(dtv->counter_defrag_ipv6_timeouts,
-                        tv->sc_perf_pca);
-                }
-            }
-            return;
-        }
-
-        next = HashListTableGetListNext(next);
-    }
-}
-
 /**
  * \brief Get the defrag policy based on the destination address of
  * the packet.
@@ -1054,7 +744,7 @@ DefragTimeoutTracker(ThreadVars *tv, DecodeThreadVars *dtv, DefragContext *dc,
  *
  * \retval The defrag policy to use.
  */
-static uint8_t
+uint8_t
 DefragGetOsPolicy(Packet *p)
 {
     int policy = -1;
@@ -1114,57 +804,19 @@ DefragGetOsPolicy(Packet *p)
     }
 }
 
+/** \internal
+ *
+ *  \retval NULL or a *LOCKED* tracker */
 static DefragTracker *
-DefragGetTracker(ThreadVars *tv, DecodeThreadVars *dtv, DefragContext *dc,
-    DefragTracker *lookup_key, Packet *p)
+DefragGetTracker(ThreadVars *tv, DecodeThreadVars *dtv, Packet *p)
 {
-    DefragTracker *tracker;
-
-    SCMutexLock(&dc->frag_table_lock);
-    tracker = HashListTableLookup(dc->frag_table, lookup_key,
-        sizeof(*lookup_key));
-    if (tracker == NULL) {
-        SCMutexLock(&dc->tracker_pool_lock);
-        tracker = PoolGet(dc->tracker_pool);
-        if (tracker == NULL) {
-            /* Timeout trackers and try again. */
-            DefragTimeoutTracker(tv, dtv, dc, p);
-            tracker = PoolGet(dc->tracker_pool);
-        }
-        SCMutexUnlock(&dc->tracker_pool_lock);
-        if (tracker == NULL) {
-            SCPerfCounterIncr(dtv->counter_defrag_max_hit,
-                tv->sc_perf_pca);
-            goto done;
-        }
-        DefragTrackerReset(tracker);
-        tracker->af = lookup_key->af;
-        tracker->id = lookup_key->id;
-        tracker->src_addr = lookup_key->src_addr;
-        tracker->dst_addr = lookup_key->dst_addr;
-        tracker->policy = DefragGetOsPolicy(p);
-
-        if (HashListTableAdd(dc->frag_table, tracker, HASHLIST_NO_SIZE) != 0) {
-            /* Failed to add new tracker. */
-            SCLogError(SC_ERR_MEM_ALLOC,
-                "Defrag: Failed to add new tracker to hash table.");
-            SCMutexLock(&dc->tracker_pool_lock);
-            PoolReturn(dc->tracker_pool, tracker);
-            SCMutexUnlock(&dc->tracker_pool_lock);
-            goto done;
-        }
-    }
-
-done:
-    SCMutexUnlock(&dc->frag_table_lock);
-    return tracker;
+    return DefragGetTrackerFromHash(p);
 }
 
 /**
  * \brief Entry point for IPv4 and IPv6 fragments.
  *
  * \param tv ThreadVars for the calling decoder.
- * \param dc A DefragContext to use, may be NULL for the default.
  * \param p The packet fragment.
  *
  * \retval A new Packet resembling the re-assembled packet if the most
@@ -1172,30 +824,22 @@ done:
  *     NULL is returned.
  */
 Packet *
-Defrag(ThreadVars *tv, DecodeThreadVars *dtv, DefragContext *dc, Packet *p)
+Defrag(ThreadVars *tv, DecodeThreadVars *dtv, Packet *p)
 {
     uint16_t frag_offset;
     uint8_t more_frags;
-    DefragTracker *tracker, lookup;
-    uint32_t id;
+    DefragTracker *tracker;
     int af;
 
-    /* If no DefragContext was passed in, use the global one.  Passing
-     * one in is primarily useful for unit tests. */
-    if (dc == NULL)
-        dc = defrag_context;
-
     if (PKT_IS_IPV4(p)) {
         af = AF_INET;
         more_frags = IPV4_GET_MF(p);
         frag_offset = IPV4_GET_IPOFFSET(p);
-        id = IPV4_GET_IPID(p);
     }
     else if (PKT_IS_IPV6(p)) {
         af = AF_INET6;
         frag_offset = IPV6_EXTHDR_GET_FH_OFFSET(p);
         more_frags = IPV6_EXTHDR_GET_FH_FLAG(p);
-        id = IPV6_EXTHDR_GET_FH_ID(p);
     }
     else {
         return NULL;
@@ -1216,28 +860,24 @@ Defrag(ThreadVars *tv, DecodeThreadVars *dtv, DefragContext *dc, Packet *p)
         }
     }
 
-    /* Create a lookup key. */
-    lookup.af = af;
-    lookup.id = id;
-    lookup.src_addr = p->src;
-    lookup.dst_addr = p->dst;
-
-    tracker = DefragGetTracker(tv, dtv, dc, &lookup, p);
+    /* return a locked tracker or NULL */
+    tracker = DefragGetTracker(tv, dtv, p);
     if (tracker == NULL)
         return NULL;
 
-    return DefragInsertFrag(tv, dtv, dc, tracker, p);
+    Packet *rp = DefragInsertFrag(tv, dtv, tracker, p);
+    DefragTrackerRelease(tracker);
+
+    return rp;
 }
 
 void
 DefragInit(void)
 {
-    /* Initialize random value for hashing and hash table size. */
-    unsigned int seed = RandomTimePreseed();
-    /* set defaults */
-    defrag_hash_rand = (int)( DEFAULT_DEFRAG_HASH_SIZE * (rand_r(&seed) / RAND_MAX + 1.0));
-
-    defrag_hash_size = DEFAULT_DEFRAG_HASH_SIZE;
+    intmax_t tracker_pool_size;
+    if (!ConfGetInt("defrag.trackers", &tracker_pool_size)) {
+        tracker_pool_size = DEFAULT_DEFRAG_HASH_SIZE;
+    }
 
     /* Allocate the DefragContext. */
     defrag_context = DefragContextNew();
@@ -1246,9 +886,12 @@ DefragInit(void)
             "Failed to allocate memory for the Defrag module.");
         exit(EXIT_FAILURE);
     }
+
+    DefragInitConfig(FALSE);
 }
 
 void DefragDestroy(void) {
+    DefragHashShutdown();
     DefragContextDestroy(defrag_context);
     defrag_context = NULL;
 }
@@ -1271,7 +914,7 @@ BuildTestPacket(uint16_t id, uint16_t off, int mf, const char content,
     IPV4Hdr ip4h;
 
     p = SCCalloc(1, sizeof(*p) + default_packet_size);
-    if (p == NULL)
+    if (unlikely(p == NULL))
         return NULL;
 
     PACKET_INITIALIZE(p);
@@ -1300,7 +943,7 @@ BuildTestPacket(uint16_t id, uint16_t off, int mf, const char content,
     SET_IPV4_DST_ADDR(p, &p->dst);
 
     pcontent = SCCalloc(1, content_len);
-    if (pcontent == NULL)
+    if (unlikely(pcontent == NULL))
         return NULL;
     memset(pcontent, content, content_len);
     PacketCopyDataOffset(p, hlen, pcontent, content_len);
@@ -1343,7 +986,7 @@ IPV6BuildTestPacket(uint32_t id, uint16_t off, int mf, const char content,
     IPV6Hdr ip6h;
 
     p = SCCalloc(1, sizeof(*p) + default_packet_size);
-    if (p == NULL)
+    if (unlikely(p == NULL))
         return NULL;
 
     PACKET_INITIALIZE(p);
@@ -1375,7 +1018,7 @@ IPV6BuildTestPacket(uint32_t id, uint16_t off, int mf, const char content,
     p->ip6eh.ip6fh->ip6fh_offlg = htons((off << 3) | mf);
 
     pcontent = SCCalloc(1, content_len);
-    if (pcontent == NULL)
+    if (unlikely(pcontent == NULL))
         return NULL;
     memset(pcontent, content, content_len);
     PacketCopyDataOffset(p, sizeof(IPV6Hdr) + sizeof(IPV6FragHdr), pcontent, content_len);
@@ -1410,7 +1053,6 @@ error:
 static int
 DefragInOrderSimpleTest(void)
 {
-    DefragContext *dc = NULL;
     Packet *p1 = NULL, *p2 = NULL, *p3 = NULL;
     Packet *reassembled = NULL;
     int id = 12;
@@ -1419,10 +1061,6 @@ DefragInOrderSimpleTest(void)
 
     DefragInit();
 
-    dc = DefragContextNew();
-    if (dc == NULL)
-        goto end;
-
     p1 = BuildTestPacket(id, 0, 1, 'A', 8);
     if (p1 == NULL)
         goto end;
@@ -1433,30 +1071,35 @@ DefragInOrderSimpleTest(void)
     if (p3 == NULL)
         goto end;
 
-    if (Defrag(NULL, NULL, dc, p1) != NULL)
+    if (Defrag(NULL, NULL, p1) != NULL)
         goto end;
-    if (Defrag(NULL, NULL, dc, p2) != NULL)
+    if (Defrag(NULL, NULL, p2) != NULL)
         goto end;
 
-    reassembled = Defrag(NULL, NULL, dc, p3);
-    if (reassembled == NULL)
+    reassembled = Defrag(NULL, NULL, p3);
+    if (reassembled == NULL) {
         goto end;
+    }
 
-    if (IPV4_GET_HLEN(reassembled) != 20)
+    if (IPV4_GET_HLEN(reassembled) != 20) {
         goto end;
-    if (IPV4_GET_IPLEN(reassembled) != 39)
+    }
+    if (IPV4_GET_IPLEN(reassembled) != 39) {
         goto end;
+    }
 
     /* 20 bytes in we should find 8 bytes of A. */
     for (i = 20; i < 20 + 8; i++) {
-        if (GET_PKT_DATA(reassembled)[i] != 'A')
+        if (GET_PKT_DATA(reassembled)[i] != 'A') {
             goto end;
+        }
     }
 
     /* 28 bytes in we should find 8 bytes of B. */
     for (i = 28; i < 28 + 8; i++) {
-        if (GET_PKT_DATA(reassembled)[i] != 'B')
+        if (GET_PKT_DATA(reassembled)[i] != 'B') {
             goto end;
+        }
     }
 
     /* And 36 bytes in we should find 3 bytes of C. */
@@ -1466,10 +1109,8 @@ DefragInOrderSimpleTest(void)
     }
 
     ret = 1;
-end:
 
-    if (dc != NULL)
-        DefragContextDestroy(dc);
+end:
     if (p1 != NULL)
         SCFree(p1);
     if (p2 != NULL)
@@ -1489,7 +1130,6 @@ end:
 static int
 DefragReverseSimpleTest(void)
 {
-    DefragContext *dc = NULL;
     Packet *p1 = NULL, *p2 = NULL, *p3 = NULL;
     Packet *reassembled = NULL;
     int id = 12;
@@ -1498,10 +1138,6 @@ DefragReverseSimpleTest(void)
 
     DefragInit();
 
-    dc = DefragContextNew();
-    if (dc == NULL)
-        goto end;
-
     p1 = BuildTestPacket(id, 0, 1, 'A', 8);
     if (p1 == NULL)
         goto end;
@@ -1512,12 +1148,12 @@ DefragReverseSimpleTest(void)
     if (p3 == NULL)
         goto end;
 
-    if (Defrag(NULL, NULL, dc, p3) != NULL)
+    if (Defrag(NULL, NULL, p3) != NULL)
         goto end;
-    if (Defrag(NULL, NULL, dc, p2) != NULL)
+    if (Defrag(NULL, NULL, p2) != NULL)
         goto end;
 
-    reassembled = Defrag(NULL, NULL, dc, p1);
+    reassembled = Defrag(NULL, NULL, p1);
     if (reassembled == NULL)
         goto end;
 
@@ -1546,9 +1182,6 @@ DefragReverseSimpleTest(void)
 
     ret = 1;
 end:
-
-    if (dc != NULL)
-        DefragContextDestroy(dc);
     if (p1 != NULL)
         SCFree(p1);
     if (p2 != NULL)
@@ -1569,7 +1202,6 @@ end:
 static int
 IPV6DefragInOrderSimpleTest(void)
 {
-    DefragContext *dc = NULL;
     Packet *p1 = NULL, *p2 = NULL, *p3 = NULL;
     Packet *reassembled = NULL;
     int id = 12;
@@ -1578,10 +1210,6 @@ IPV6DefragInOrderSimpleTest(void)
 
     DefragInit();
 
-    dc = DefragContextNew();
-    if (dc == NULL)
-        goto end;
-
     p1 = IPV6BuildTestPacket(id, 0, 1, 'A', 8);
     if (p1 == NULL)
         goto end;
@@ -1592,11 +1220,11 @@ IPV6DefragInOrderSimpleTest(void)
     if (p3 == NULL)
         goto end;
 
-    if (Defrag(NULL, NULL, dc, p1) != NULL)
+    if (Defrag(NULL, NULL, p1) != NULL)
         goto end;
-    if (Defrag(NULL, NULL, dc, p2) != NULL)
+    if (Defrag(NULL, NULL, p2) != NULL)
         goto end;
-    reassembled = Defrag(NULL, NULL, dc, p3);
+    reassembled = Defrag(NULL, NULL, p3);
     if (reassembled == NULL)
         goto end;
 
@@ -1623,8 +1251,6 @@ IPV6DefragInOrderSimpleTest(void)
 
     ret = 1;
 end:
-    if (dc != NULL)
-        DefragContextDestroy(dc);
     if (p1 != NULL)
         SCFree(p1);
     if (p2 != NULL)
@@ -1664,11 +1290,11 @@ IPV6DefragReverseSimpleTest(void)
     if (p3 == NULL)
         goto end;
 
-    if (Defrag(NULL, NULL, dc, p3) != NULL)
+    if (Defrag(NULL, NULL, p3) != NULL)
         goto end;
-    if (Defrag(NULL, NULL, dc, p2) != NULL)
+    if (Defrag(NULL, NULL, p2) != NULL)
         goto end;
-    reassembled = Defrag(NULL, NULL, dc, p1);
+    reassembled = Defrag(NULL, NULL, p1);
     if (reassembled == NULL)
         goto end;
 
@@ -1712,7 +1338,6 @@ DefragDoSturgesNovakTest(int policy, u_char *expected, size_t expected_len)
 {
     int i;
     int ret = 0;
-    DefragContext *dc = NULL;
 
     DefragInit();
 
@@ -1783,14 +1408,11 @@ DefragDoSturgesNovakTest(int policy, u_char *expected, size_t expected_len)
     /* Q*16 at 176. */
     packets[16] = BuildTestPacket(id, 176 >> 3, 0, 'Q', 16);
 
-    dc = DefragContextNew();
-    if (dc == NULL)
-        goto end;
     default_policy = policy;
 
     /* Send all but the last. */
     for (i = 0; i < 9; i++) {
-        Packet *tp = Defrag(NULL, NULL, dc, packets[i]);
+        Packet *tp = Defrag(NULL, NULL, packets[i]);
         if (tp != NULL) {
             SCFree(tp);
             goto end;
@@ -1801,7 +1423,7 @@ DefragDoSturgesNovakTest(int policy, u_char *expected, size_t expected_len)
     }
     int overlap = 0;
     for (; i < 16; i++) {
-        Packet *tp = Defrag(NULL, NULL, dc, packets[i]);
+        Packet *tp = Defrag(NULL, NULL, packets[i]);
         if (tp != NULL) {
             SCFree(tp);
             goto end;
@@ -1810,35 +1432,35 @@ DefragDoSturgesNovakTest(int policy, u_char *expected, size_t expected_len)
             overlap++;
         }
     }
-    if (!overlap)
+    if (!overlap) {
         goto end;
+    }
 
     /* And now the last one. */
-    Packet *reassembled = Defrag(NULL, NULL, dc, packets[16]);
-    if (reassembled == NULL)
+    Packet *reassembled = Defrag(NULL, NULL, packets[16]);
+    if (reassembled == NULL) {
         goto end;
+    }
 
-    if (IPV4_GET_HLEN(reassembled) != 20)
+    if (IPV4_GET_HLEN(reassembled) != 20) {
         goto end;
-    if (IPV4_GET_IPLEN(reassembled) != 20 + 192)
+    }
+    if (IPV4_GET_IPLEN(reassembled) != 20 + 192) {
         goto end;
+    }
 
-    if (memcmp(GET_PKT_DATA(reassembled) + 20, expected, expected_len) != 0)
+    if (memcmp(GET_PKT_DATA(reassembled) + 20, expected, expected_len) != 0) {
         goto end;
+    }
     SCFree(reassembled);
 
-    /* Make sure the tracker was released back to the pool. */
-    if (dc->tracker_pool->outstanding != 0)
-        return 0;
-
     /* Make sure all frags were returned back to the pool. */
-    if (dc->frag_pool->outstanding != 0)
-        return 0;
+    if (defrag_context->frag_pool->outstanding != 0) {
+        goto end;
+    }
 
     ret = 1;
 end:
-    if (dc != NULL)
-        DefragContextDestroy(dc);
     for (i = 0; i < 17; i++) {
         SCFree(packets[i]);
     }
@@ -1851,7 +1473,6 @@ IPV6DefragDoSturgesNovakTest(int policy, u_char *expected, size_t expected_len)
 {
     int i;
     int ret = 0;
-    DefragContext *dc = NULL;
 
     DefragInit();
 
@@ -1922,14 +1543,11 @@ IPV6DefragDoSturgesNovakTest(int policy, u_char *expected, size_t expected_len)
     /* Q*16 at 176. */
     packets[16] = IPV6BuildTestPacket(id, 176 >> 3, 0, 'Q', 16);
 
-    dc = DefragContextNew();
-    if (dc == NULL)
-        goto end;
     default_policy = policy;
 
     /* Send all but the last. */
     for (i = 0; i < 9; i++) {
-        Packet *tp = Defrag(NULL, NULL, dc, packets[i]);
+        Packet *tp = Defrag(NULL, NULL, packets[i]);
         if (tp != NULL) {
             SCFree(tp);
             goto end;
@@ -1940,7 +1558,7 @@ IPV6DefragDoSturgesNovakTest(int policy, u_char *expected, size_t expected_len)
     }
     int overlap = 0;
     for (; i < 16; i++) {
-        Packet *tp = Defrag(NULL, NULL, dc, packets[i]);
+        Packet *tp = Defrag(NULL, NULL, packets[i]);
         if (tp != NULL) {
             SCFree(tp);
             goto end;
@@ -1953,7 +1571,7 @@ IPV6DefragDoSturgesNovakTest(int policy, u_char *expected, size_t expected_len)
         goto end;
 
     /* And now the last one. */
-    Packet *reassembled = Defrag(NULL, NULL, dc, packets[16]);
+    Packet *reassembled = Defrag(NULL, NULL, packets[16]);
     if (reassembled == NULL)
         goto end;
     if (memcmp(GET_PKT_DATA(reassembled) + 40, expected, expected_len) != 0)
@@ -1964,18 +1582,15 @@ IPV6DefragDoSturgesNovakTest(int policy, u_char *expected, size_t expected_len)
 
     SCFree(reassembled);
 
-    /* Make sure the tracker was released back to the pool. */
-    if (dc->tracker_pool->outstanding != 0)
-        return 0;
-
     /* Make sure all frags were returned to the pool. */
-    if (dc->frag_pool->outstanding != 0)
-        return 0;
+    if (defrag_context->frag_pool->outstanding != 0) {
+        printf("defrag_context->frag_pool->outstanding %u: ", defrag_context->frag_pool->outstanding);
+        goto end;
+    }
 
     ret = 1;
+
 end:
-    if (dc != NULL)
-        DefragContextDestroy(dc);
     for (i = 0; i < 17; i++) {
         SCFree(packets[i]);
     }
@@ -2401,9 +2016,6 @@ DefragTimeoutTest(void)
 {
     int i;
     int ret = 0;
-    DefragContext *dc = NULL;
-
-    DefragInit();
 
     /* Setup a small numberr of trackers. */
     if (ConfSet("defrag.trackers", "16", 1) != 1) {
@@ -2411,9 +2023,7 @@ DefragTimeoutTest(void)
         goto end;
     }
 
-    dc = DefragContextNew();
-    if (dc == NULL)
-        goto end;
+    DefragInit();
 
     /* Load in 16 packets. */
     for (i = 0; i < 16; i++) {
@@ -2421,7 +2031,7 @@ DefragTimeoutTest(void)
         if (p == NULL)
             goto end;
 
-        Packet *tp = Defrag(NULL, NULL, dc, p);
+        Packet *tp = Defrag(NULL, NULL, p);
 
         SCFree(p);
 
@@ -2437,39 +2047,25 @@ DefragTimeoutTest(void)
     if (p == NULL)
         goto end;
 
-    p->ts.tv_sec += (dc->timeout + 1);
-    Packet *tp = Defrag(NULL, NULL, dc, p);
-
-    SCFree(p);
+    p->ts.tv_sec += (defrag_context->timeout + 1);
+    Packet *tp = Defrag(NULL, NULL, p);
 
     if (tp != NULL) {
         SCFree(tp);
         goto end;
     }
 
-    /* Iterate our HashList and look for the trackerr with id 99. */
-    int found = 0;
-    HashListTableBucket *next = HashListTableGetListHead(dc->frag_table);
-    if (next == NULL)
+    DefragTracker *tracker = DefragLookupTrackerFromHash(p);
+    if (tracker == NULL)
         goto end;
-    for (;;) {
-        if (next == NULL)
-            break;
-        DefragTracker *tracker = HashListTableGetListData(next);
-        if (tracker->id == 99) {
-            found = 1;
-            break;
-        }
 
-        next = HashListTableGetListNext(next);
-    }
-    if (found == 0)
+    if (tracker->id != 99)
         goto end;
 
+    SCFree(p);
+
     ret = 1;
 end:
-    if (dc != NULL)
-        DefragContextDestroy(dc);
     DefragDestroy();
     return ret;
 }
@@ -2500,7 +2096,7 @@ DefragIPv4NoDataTest(void)
         goto end;
 
     /* We do not expect a packet returned. */
-    if (Defrag(NULL, NULL, dc, p) != NULL)
+    if (Defrag(NULL, NULL, p) != NULL)
         goto end;
 
     /* The fragment should have been ignored so no fragments should
@@ -2539,7 +2135,7 @@ DefragIPv4TooLargeTest(void)
         goto end;
 
     /* We do not expect a packet returned. */
-    if (Defrag(NULL, NULL, dc, p) != NULL)
+    if (Defrag(NULL, NULL, p) != NULL)
         goto end;
     if (!ENGINE_ISSET_EVENT(p, IPV4_FRAG_PKT_TOO_LARGE))
         goto end;
