@@ -53,6 +53,8 @@
 #include "detect-engine-hsmd.h"
 #include "detect-engine-hscd.h"
 #include "detect-engine-hua.h"
+#include "detect-engine-hhhd.h"
+#include "detect-engine-hrhhd.h"
 #include "detect-engine-file.h"
 
 #include "detect-engine.h"
@@ -200,6 +202,18 @@ void DetectEngineRegisterAppInspectionEngines(void)
           DE_STATE_FLAG_HUAD_MATCH,
           0,
           DetectEngineInspectHttpUA },
+        { ALPROTO_HTTP,
+          DETECT_SM_LIST_HHHDMATCH,
+          DE_STATE_FLAG_HHHD_INSPECT,
+          DE_STATE_FLAG_HHHD_MATCH,
+          0,
+          DetectEngineInspectHttpHH },
+        { ALPROTO_HTTP,
+          DETECT_SM_LIST_HRHHDMATCH,
+          DE_STATE_FLAG_HRHHD_INSPECT,
+          DE_STATE_FLAG_HRHHD_MATCH,
+          0,
+          DetectEngineInspectHttpHRH },
     };
 
     struct tmp_t data_toclient[] = {
@@ -507,6 +521,7 @@ static void *DetectEngineLiveRuleSwap(void *arg)
                 if (detect_tvs[i]->inq != NULL) {
                     Packet *p = PacketGetFromAlloc();
                     if (p != NULL) {
+                        p->flags |= PKT_PSEUDO_STREAM_END;
                         PacketQueue *q = &trans_q[detect_tvs[i]->inq->id];
                         SCMutexLock(&q->mutex_q);
 
